#define LIBDPKG_VOLATILE_API
#include <dpkg/dpkg.h>
#include <dpkg/dpkg-db.h>
#include <dpkg/pkg-array.h>
#include "dpkg.h"

/*
http://dpkg.alioth.debian.org/doc/
*/

int read_dpkg_header(vector<string>& packages)
{
	return -1;
}
int read_dpkg_items(vector<string>& dpkg)
{
	return -1;
}
